#include "view/main_window.h"
#include "view/hover_window.h"
#include "$(appName).h"

#define FILE_PATH_MAX_LEN 256
#define SOURCE_THEME_EXTENSION	"edje/sources_ext.edj"
#define SINK_IMAGE	"images/sink.png"


typedef enum {
	CONTROLLER_STATE_NORMAL = 0,
	CONTROLLER_STATE_PRESS,
	CONTROLLER_STATE_DRAG
}
controller_state_t;

static struct {
	controller_state_t state;
	int drag_start_x;
	int drag_start_y;

} s_controller_data = {
	.state = CONTROLLER_STATE_NORMAL,
	.drag_start_x = 0,
	.drag_start_y = 0
};

static void
__attach_callbacks(void);

static Eina_Bool
__press_cb(void *data, int type, void *event);

static Eina_Bool
__move_cb(void *data, int type, void *event);

static Eina_Bool
__unpress_cb(void *data, int type, void *event);

static void
__resource_path_compile(const char *edj_file_name, char *full_path_out);


bool
create_application_view(void)
{
	char full_path_source_extension[FILE_PATH_MAX_LEN] = {0,};
	char full_path_sink_image[FILE_PATH_MAX_LEN] = {0,};

	__resource_path_compile(SOURCE_THEME_EXTENSION, full_path_source_extension);
	__resource_path_compile(SINK_IMAGE, full_path_sink_image);


	if (!create_main_window()) {
		dlog_print(DLOG_ERROR, LOG_TAG, "View creation failed: on window handle");
		return false;
	}
	if (!create_main_window_source_wgt(full_path_source_extension)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "View creation failed: on source widget");
		return false;
	}
	if (!create_main_window_sink_entry_wgt(full_path_sink_image)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "View creation failed: on sink widget");
		return false;
	}
	if (!create_main_window_hover(full_path_source_extension)) {
		dlog_print(DLOG_ERROR, LOG_TAG, "View creation failed: on hover");
		return false;
	}

	__attach_callbacks();

	dlog_print(DLOG_DEBUG, LOG_TAG, "View creation successful");

	return true;
}

static Eina_Bool
__press_cb(void *data, int type, void *event)
{
	Ecore_Event_Mouse_Button *press = (Ecore_Event_Mouse_Button *)event;

	if (!press) {
		dlog_print(DLOG_ERROR, LOG_TAG, "press cb error");
		return EINA_TRUE;
	}
	/* state test && hit test: */
	if (!(s_controller_data.state == CONTROLLER_STATE_NORMAL &&
		test_main_window_source_wgt_hit(press->root.x, press->root.y))
	)
		return EINA_TRUE;

	/* paste text from clipboard to hover: */
	paste_main_window_hover_text();

	s_controller_data.state = CONTROLLER_STATE_PRESS;
	s_controller_data.drag_start_x = press->root.x;
	s_controller_data.drag_start_y = press->root.y;

	_set_hover_window_coordinates(press->root.x, press->root.y);
	_show_hover_window();

	return EINA_TRUE;
}

static Eina_Bool
__move_cb(void *data, int type, void *event)
{
	Ecore_Event_Mouse_Move *move = (Ecore_Event_Mouse_Move *)event;

	if (!move) {
		dlog_print(DLOG_ERROR, LOG_TAG, "move cb error");
		return EINA_TRUE;
	}

	if (s_controller_data.state == CONTROLLER_STATE_PRESS)
		s_controller_data.state = CONTROLLER_STATE_DRAG;

	if (s_controller_data.state != CONTROLLER_STATE_DRAG)
		return EINA_TRUE;

	_set_hover_window_coordinates(move->root.x, move->root.y);

	return EINA_TRUE;
}

static Eina_Bool
__unpress_cb(void *data, int type, void *event)
{
	Ecore_Event_Mouse_Button *unpress = (Ecore_Event_Mouse_Button *)event;

	if (!unpress) {
		dlog_print(DLOG_ERROR, LOG_TAG, "unpress cb error");
		return EINA_TRUE;
	}

	_hide_hover_window();

	/* state && hit test */
	if (s_controller_data.state == CONTROLLER_STATE_DRAG &&
		test_main_window_sink_wgt_hit(unpress->root.x, unpress->root.y))
		paste_window_sink_wgt_text();

	 s_controller_data.state = CONTROLLER_STATE_NORMAL;

	return EINA_TRUE;
}

static void
__attach_callbacks(void)
{
	ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_DOWN, __press_cb, NULL);
	ecore_event_handler_add(ECORE_EVENT_MOUSE_MOVE, __move_cb, NULL);
	ecore_event_handler_add(ECORE_EVENT_MOUSE_BUTTON_UP, __unpress_cb, NULL);
}

static void
__resource_path_compile(const char *edj_file_name, char *full_path_out)
{
	char *res_path = app_get_resource_path();
	if (res_path) {
		snprintf(full_path_out, FILE_PATH_MAX_LEN, "%s%s", res_path, edj_file_name);
		free(res_path);
	}
}

void
pause_application(void)
{
	s_controller_data.state = CONTROLLER_STATE_NORMAL;
}

void
terminate_application(void)
{
	/* detach callbacks: */
	ecore_event_handler_del((Ecore_Event_Handler *)__press_cb);
	ecore_event_handler_del((Ecore_Event_Handler *)__move_cb);
	ecore_event_handler_del((Ecore_Event_Handler *)__unpress_cb);
	destroy_main_window();
}
